/**
 * Created by christiankalig on 05.07.16.
 */
var map;
var app = app || {};

var JobReview = (function(){
    var instance;

    function init(){
        // private

        function showMap(){
            map = plugin.google.maps.Map.getMap({
                'backgroundColor': 'white',
                'mapType': plugin.google.maps.MapTypeId.HYBRID,
                'controls': {
                    'compass': false,
                    'myLocationButton': false,
                    'indoorPicker': false,
                    'zoom': true
                },
                'gestures': {
                    'scroll': true,
                    'tilt': false,
                    'rotate': false,
                    'zoom': true
                }
            });
            map.addEventListener(plugin.google.maps.event.MAP_READY, onMapReady);
        }

        function onMapReady(){
        	if (!app.data.currentTrip) {
        		app.utils.debug.log(app.utils.debug.debugPages.JOB_REVIEW, "no trip selected");
                console.log("no trip selected");
        		return;
        	}
            app.utils.debug.log(app.utils.debug.debugPages.JOB_REVIEW, "Map loaded");
            /***
             * Load starting point, add marker and animate the camera
             */
            $('#job-review-show-map').click(function(event){
                map.showDialog();
            });

            /**
             * load the first position of the trip and add a marker to the map
             */
            app.db.executeSql("SELECT posJsonObj FROM position WHERE trip_id = ? ORDER BY id ASC LIMIT 1", [app.data.currentTrip], function(result){
                var position = $.parseJSON(result.rows.item(0).posJsonObj);
                console.log(result.rows.item(0).posJsonObj);
                var latLng = new plugin.google.maps.LatLng(position.latitude, position.longitude);
                console.log(latLng);
                map.animateCamera({
                    'target': latLng,
                    'zoom': 17,
                    'duration': 2500
                });
                map.addMarker({
                    'position': latLng,
                    'title': 'Beginn',
                    zIndex: 999
                }, function(marker){
                    console.log("begin marker added");
                    marker.showInfoWindow();
                });
            });

            /**
             * Load all positions regarding this trip and draw polyline on map
             */
            app.db.executeSql("SELECT posJsonObj FROM position WHERE trip_id = ?", [app.data.currentTrip], function(result){
                var posLatLng = [], item;
                for(var i = 0; i < result.rows.length; i++){
                    item = result.rows.item(i);
                    var position = $.parseJSON(item.posJsonObj);
                    console.log(position);
                    posLatLng.push(new plugin.google.maps.LatLng(position.latitude, position.longitude));
                }
                map.addPolyline({
                    points: posLatLng,
                    'color': '#4C4CFF',
                    'width': '10',
                    'geodesic': true,
                    zIndex: 999
                });
            });
            /**
             * Load last position and add a marker to the map
             */
            app.db.executeSql("SELECT posJsonObj FROM position WHERE trip_id = ? ORDER BY id DESC LIMIT 1", [app.data.currentTrip], function(result){
                var position = $.parseJSON(result.rows.item(0).posJsonObj);
                map.addMarker({
                    'position': new plugin.google.maps.LatLng(position.latitude, position.longitude),
                    'title': 'Ende',
                    zIndex: 999
                }, function(marker){
                    marker.showInfoWindow();
                });
            });
            app.utils.debug.log(app.utils.debug.debugPages.JOB_REVIEW, app.data.currentTrip);
            /**
             * load all occasions (images and notes) and sort them by creation time
             */
            app.db.executeSql("SELECT * FROM image WHERE trip_id = ?", [app.data.currentTrip], function(result){
                var occasions = [], item;
                for(var i = 0; i < result.rows.length; i++){
                    item = result.rows.item(i);
                    occasions.push([item.created, item]);
                }
                app.db.executeSql("SELECT * FROM note WHERE trip_id = ?", [app.data.currentTrip], function(result1){
                    var item1;
                    for(var i = 0; i < result1.rows.length; i++){
                        item1 = result1.rows.item(i);
                        occasions.push([item1.created, item1]);
                    }
                    occasions.sort(function(a, b){
                        a = a[0];
                        b = b[0];
                        return a < b ? -1 : (a > b ? 1 : 0);
                    });
                    generateReviewTimeline(occasions);
                });
            });

            //var distance = calculateDistance(trip.positions);
            //var time = app.utils.timeIntervalToString(trip.begin, trip.end);
            //$('#job-review-map-time').text(0);
            //$('#job-review-map-distance').text(0+" m");
            //app.pages.jobReview.generateReviewTimeline();
            $.mobile.changePage('#job-review');
        }

        function calculateDistance (positions){
            var i, distance = 0, ele, nextele;
            for(i = 0; i < positions.length-1; i++){
                ele = positions[i];
                nextele = positions[i+1];
                distance += app.utils.measure(ele.coords.latitude, ele.coords.longitude, nextele.coords.latitude, nextele.coords.longitude);
            }
            return app.utils.round(distance, 2);
        }

        function generateReviewTimeline(occasions){
            var ref = $('#job-review-tab-timeline');
            ref.empty();
            ref.append(Timeline.generateStartPoint());
            ref.append(Timeline.generateDivider(50));
            console.log(occasions);
            $.each(occasions, function(key, value){
                if(value[1].uri){
                    ref.append(Timeline.generateImageItem(value[1].uri));
                } else {
                    ref.append(Timeline.generateNoteItem(value[1].note));
                }
                ref.append(Timeline.generateDivider(50));
            });
            ref.append(Timeline.generateEndPoint());
        }

        var Timeline = {
            generateStartPoint: function(){
                return '' +
                    '<div class="row">' +
                    '   <div class="col-xs-8 col-xs-offset-2 job-review-item job-review-start">' +
                    '       <h4 class="text-center">Start</h4>' +
                    '   </div>' +
                    '</div>';
            },
            generateEndPoint: function(){
                return '' +
                    '<div class="row">' +
                    '   <div class="col-xs-8 col-xs-offset-2 job-review-item job-review-end">' +
                    '       <h4 class="text-center">Ende</h4>' +
                    '   </div>' +
                    '</div>';
            },
            generateDivider: function(distance){
                return '' +
                    '<div class="row">' +
                    '   <div class="col-xs-8 col-xs-offset-2 job-review-divider">' +
                    '       <div></div>' +
                    '   </div>' +
                    '</div>';
            },
            generateNoteItem: function(text){
                return '' +
                    '<div class="row">' +
                    '   <div class="col-xs-10 col-xs-offset-1 job-review-item">' +
                    '       <h4>Notiz</h4>' +
                    '       <p>'+text+'</p>' +
                    '   </div>' +
                    '</div>';
            },
            generateImageItem: function(uri){
                return '' +
                    '<div class="row">' +
                    '   <div class="col-xs-10 col-xs-offset-1 job-review-item">' +
                    '       <h4>Bild</h4>' +
                    '       <img src="'+uri+'" />' +
                    '   </div>' +
                    '</div>';
            }
        };

        function hideMap(){
            map.remove();
        }

        //public
        return {
            var_public: "public",
            onCreate: function(){

            },
            beforeShow: function(){
                showMap();
            },
            beforeHide: function(){
                hideMap();
            },
            setClickable: function(clickable){
                if(map != null){
                    map.setClickable(clickable);
                }
            },
            showMap: function(){
                showMap();
            }
        }
    }

    return {
        getInstance: function(){
            if(!instance){
                instance = init();
            }
            return instance;
        }
    }
})();
