/**
 * Created by christiankalig on 05.07.16.
 */
var app = app || {};

var JobTracker = (function(){
    var instance;

    function init(){
    	console.log("jobTracker init called");
        // private
        // references
        var trackBtn = $('#job-tracker-start-tracking'),
            takePicBtn = $('#job-tracker-take-pic'),
            addNoteBtn = $('#job-tracker-add-note'),
            loaderRef = $('.loader'),
            currentDistanceRef = $('#job-tracker-current-distance'),
            currentTimeRef = $('#job-tracker-current-time'),
            imageCounterRef = $('#job-tracker-image-counter'),
            noteCounterRef = $('#job-tracker-note-counter');

        // variables
        var isTracking = false,
            trip = null,
            currentDistance = 0,
            currentTime = 0,
            timeInterval = 0,
            lastPosition = null,
            noteCount = 0,
            imageCount = 0,
            trackBegin = null,
            trackEnd = null,
            watchId = null,
            hasValidPosition = false,
            showProtocol = true,
            startPositionLogged = false;

        //camera options
        var cameraOptions = {
            // Some common settings are 20, 50, and 100
            quality: 20,
            destinationType: Camera.DestinationType.NATIVE_URI,
            // In this app, dynamically set the picture source, Camera or photo gallery
            sourceType: Camera.PictureSourceType.CAMERA,
            encodingType: Camera.EncodingType.JPEG,
            mediaType: Camera.MediaType.PICTURE,
            allowEdit: false,
            correctOrientation: true  //Corrects Android orientation quirks
        };

        $(document).on('start_tracking', function(e) {
            $('.logo_bg').addClass('trip_active');
		});

        $(document).on('stop_tracking', function(e) {
            $('.logo_bg').removeClass('trip_active');
		});

        trackBtn.click(function(event, ui){
            if(isTracking){
                confirmStopTracking();
            } else {
                startTracking();
            }
        });
        
        $('.docs-toggle-class').click(function() {
        	$('#docs-toggle').toggle(400);
        });
        
        /**
         * called for asking whether or not a trip should really be finished
         */
        function confirmEndTrip() {
        	navigator.notification.confirm("Fahrt wirklich beenden?", stopTrip, "Achtung", ["OK", "Abbrechen"]);
        }

        /**
         * called for stopping a trip. If tracking is currently running, tracking will be stopped as well.
         */
        function stopTrip(index) {
        	if (index == 2) {
        		return;
        	}
        	$('.logo_bg').removeClass('trip_active');
        	if (isTracking) {
        		//end tracking first
        		stopTracking(1);
        	}
        	//send API call ending trip
        	stopTracking(3);
        }

        /**
         * asks whether tracking should be stopped
         */
        function confirmStopTracking() {
        	navigator.notification.confirm("Aufzeichnung wirklich beenden?", stopTracking, "Achtung", ["OK", "Abbrechen"]);
        }

        takePicBtn.click(function(event, ui){
            takePicture();
        });

        // showMapBtn.click(function(event, ui){
        //     showMap();
        // });

        $('#job-tracker-bottomsheet-note-btn').click(function(event){
            var text = $('#job-tracker-bottomsheet-note-textarea').val();
            addNote(text);
        });

        function reset(){
            currentDistanceRef.text("0 m");
            currentDistance = 0;
            currentTimeRef.text("00:00:00");
            currentTime = 0;
            imageCounterRef.html("<span>0</span>Bilder");
            imageCount = 0;
            noteCounterRef.html("<span>0</span>Notizen");
            noteCount = 0;
            $('#job-tracker-signal-alert').show();
        	$('#job-tracker-current-distance').hide();
        	$('#job-tracker-accuracy').html(0);
        	lastPosition = null;
        	$('#job-tracker-speed').html(0);
        	$('#job-tracker-speed-div').hide();
        }

        function startTracking(){
        	if (app.data.currentTripExtId != null && app.data.currentTripExtId > 0) {
	            $(document).trigger("start_tracking");
	            app.data.isTracking = true;
	            isTracking = true;
	            trackBegin = new Date();
	            watchId = navigator.geolocation.watchPosition(onTrackingSuccess, onTrackingError, { timeout: Infinity, enableHighAccuracy: true, maximumAge: 0 });
	            trackBtn.text("Aufzeichnung beenden");
	            showLoadingAnimation();
	            timeInterval = window.setInterval(function(){
	                updateLayout();
	            }, 1000);
	            window.plugins.insomnia.keepAwake(function(){app.utils.debug.log(app.utils.debug.debugPages.JOB_TRACKER, "no sleep enabled")}, function(){app.utils.debug.log(app.utils.debug.debugPages.JOB_TRACKER, "keep awake not working")});
        	}
        	else {
        		alert('Tracking kann nicht gestartet werden, da es noch keine aktive Fahrt gibt.');
        	}
        }

        var ignorePosition = null;
        var ignoreMeters = 0;

        /**
         * saves a position in the database
         */
        function savePosition(position, meters) {
        	lastPosition = position;
            var pos = {};
            pos["timestamp"] = position.timestamp;
            pos["longitude"] = position.coords.longitude;
            pos["latitude"] = position.coords.latitude;
            pos["accuracy"] = position.coords.accuracy;
            pos["altitude"] = position.coords.altitude;
            pos["altitudeAccuracy"] = position.coords.altitudeAccuracy;
            pos["heading"] = position.coords.heading;
            pos["speed"] = position.coords.speed;
            var posJsonObj = JSON.stringify(pos);
            console.log("posJsonObj: ");
            console.log(posJsonObj);
            console.log(position);
            app.db.executeSql("INSERT INTO position (trip_id, posJsonObj) VALUES (?, ?)", [app.data.currentTrip, posJsonObj]);
            currentDistance += meters;
            ignorePosition = null;
    		ignoreMeters = 0;
        }

        function toRad(val) {
        	return val * 0.017453292519943295; // 0.017453292519943295 === Math.PI / 180
        }
        
        function set_accuracy_info(type) {
        	$('#job-tracker-signal-alert').hide();
        	$('#job-tracker-signal-warning').hide();
        	$('#job-tracker-current-distance').hide();
        	$('#job-tracker-current-signal-ok').hide();
        	$('#job-tracker-current-signal-warning').hide();
        	$('#job-tracker-current-signal-alert').hide();
        	if (type == "alert") {
        		$('#job-tracker-signal-alert').show();
        		$('#job-tracker-current-signal-alert').show();
        	}
        	else if (type == "warning") {
        		$('#job-tracker-signal-warning').show();
        		$('#job-tracker-current-signal-warning').show();
        	}
        	else {
        		$('#job-tracker-current-signal-ok').show();
        		$('#job-tracker-current-distance').show();
        	}
        }
        
        function onTrackingSuccess(position){
        	if (app.data.currentTripExtId != null && app.data.currentTripExtId > 0) {
        		console.log("accuracy: " + position.coords.accuracy);
        		$('#job-tracker-accuracy').html(Math.round(position.coords.accuracy));
        		if ((lastPosition == null && position.coords.accuracy > 20) ||
        			(lastPosition != null && position.coords.accuracy > 75)){
        			set_accuracy_info("alert");
        			app.utils.debug.log(app.utils.debug.debugPages.JOB_TRACKER, "don't use position with accuracy: "+position.coords.accuracy, "INFO", app.data.currentTripExtId);
        			hasValidPosition = false;
        			hideLoadingAnimation();
        			return;
        		}
        		hasValidPosition = true;
        		showLoadingAnimation();
        		if (position.coords.accuracy > 25) {
        			set_accuracy_info("warning");
        		}
        		else {
        			set_accuracy_info("ok");
        			if (!startPositionLogged) {
        				app.utils.debug.log(app.utils.debug.debugPages.JOB_TRACKER, "tracking start", "INFO", app.data.currentTripExtId);
        				app.utils.debug.log(app.utils.debug.debugPages.JOB_TRACKER, 
        					"{'lat':"+position.coords.latitude+",'lon':"+position.coords.longitude+"'}", "INFO", app.data.currentTripExtId);
        				startPositionLogged = true;
        			}
        		}
        		var meters = 2;
	            if(lastPosition != null){
	                meters = app.utils.measure(lastPosition.coords.latitude, lastPosition.coords.longitude, position.coords.latitude, position.coords.longitude);
	                $('#job-tracker-speed-div').hide();
	                
	                var d = new Date(position.timestamp);
					var hours = d.getHours(),
					    minutes = d.getMinutes(),
					    seconds = d.getSeconds(),
					    month = d.getMonth() + 1,
					    day = d.getDate(),
					    year = d.getFullYear() % 100;
					
					function pad(d) {
					    return (d < 10 ? "0" : "") + d;
					}
					
					var formattedDate = pad(hours) + ":"
					                  + pad(minutes) + ":"
					                  + pad(seconds) + " "
					                  + pad(month) + "-"
					                  + pad(day) + "-"
					                  + pad(year);
	                
					console.log(formattedDate);
					
					d = new Date(lastPosition.timestamp);
					hours = d.getHours(),
					    minutes = d.getMinutes(),
					    seconds = d.getSeconds(),
					    month = d.getMonth() + 1,
					    day = d.getDate(),
					    year = d.getFullYear() % 100;
										
					formattedDate = pad(hours) + ":"
					                  + pad(minutes) + ":"
					                  + pad(seconds) + " "
					                  + pad(month) + "-"
					                  + pad(day) + "-"
					                  + pad(year);
	                
					console.log(formattedDate);
					
	                var diff_time = position.timestamp-lastPosition.timestamp;
	                var diff_seconds = diff_time/10000.0;
	                var speed = (meters/diff_seconds) * 3.6;
	                console.log(position.timestamp);
	                console.log(lastPosition.timestamp);
	                console.log(diff_time/10000.0);
	                console.log(meters);
	                console.log(speed + " km/h");
	                
	                if (diff_seconds < 1) {
	                	console.log("skip position < 1s");
	                	return;
	                }
	                if (meters > 2) {
	                	app.utils.debug.log(app.utils.debug.debugPages.JOB_TRACKER, 
	                		"{'lat':"+position.coords.latitude+",'lon':"+position.coords.longitude+
	                		",'meters':"+meters+",'speed':"+speed + "}", "INFO", app.data.currentTripExtId);
	                }
	            }
	            // 2017-07-04, Patric Eid: don't check for max meteres anymore, since we already check the accuracy
	            /*if (meters >= 50) {
	            	//ignore if meters is too high
	            	if (ignorePosition == null) {
	            		ignorePosition = position;
	            		ignoreMeters = meters;
	            		app.utils.debug.log(app.utils.debug.debugPages.JOB_TRACKER, "skip position: "+position);
	            	}
	            	else {
	            		app.utils.debug.log(app.utils.debug.debugPages.JOB_TRACKER, "don't use position: "+position);
	            		savePosition(ignorePosition, ignoreMeters);
	            	}
	            }
	            else*/ 
	            if(meters >= 2){
	            	savePosition(position, meters);
	            }
        	}
        	else {
        		alert('Position kann nicht gespeichert werden, da es keine aktive Fahrt gibt!');
        	}
        }

        function onTrackingError(err){
            app.utils.debug.log(app.utils.debug.debugPages.JOB_TRACKER, err, "ERROR");
        }

        function stopTracking(index){
            if(index == 2) return;
            app.utils.debug.log(app.utils.debug.debugPages.JOB_TRACKER, "stop tracking", "INFO", app.data.currentTripExtId);
            if (lastPosition != null) {
            	app.utils.debug.log(app.utils.debug.debugPages.JOB_TRACKER, lastPosition.coords, "INFO", app.data.currentTripExtId);
            	lastPosition = null;
            }
            else {
            	app.utils.debug.log(app.utils.debug.debugPages.JOB_TRACKER, "Keine Position vorhanden!", "WARN", app.data.currentTripExtId);
            }
            startPositionLogged = false;
            $(document).trigger("stop_tracking");
            isTracking = false;
            app.data.isTracking = false;
            navigator.geolocation.clearWatch(watchId);
            hasValidPosition = false;
            trackEnd = new Date();
            trackBtn.text("Aufzeichnung starten");
            //stop trip on server
            hideLoadingAnimation();
            clearInterval(timeInterval);
            window.plugins.insomnia.allowSleepAgain(function(){app.utils.debug.log(app.utils.debug.debugPages.JOB_TRACKER, "no sleep disabled", "INFO")}, function(){app.utils.debug.log(app.utils.debug.debugPages.JOB_TRACKER, "keep awake not working", "ERROR")});

            if (index == 3) {
              	//called via backbutton
              	app.pages.tripOverview.loadTripData();
               	$.mobile.changePage("#trip-overview");
             }
             else {
              	//end trip manually
    	        reset();
    	        app.data.isTracking = true; //tracking is not finished yet..
    	        console.log("showProtocol: " + showProtocol);
    	        if (showProtocol*1 > 0) {
    	        	$.mobile.changePage("#note-protocol");
    	        }
    	        else {
    	        	console.log("skip protocol");
    	        	//finish without showing protocol
    	        	 app.db.executeSql("INSERT INTO trip_ended (trip_id, job_id) VALUES (?, ?)", [app.data.currentTrip, app.data.currentJob], false,
                        function(error){alert(error);}
                     );
    	        	 app.data.isTracking = true;
    	        	 app.pages.jobReview.showMap();
    	        	 
    	        	 //TODO: testing only
    	        	 console.log("get log");
    	        	 app.utils.debug.getLog();
    	        }
             }
        }

        function takePicture(){
        	if (lastPosition == null) {
        		alert("Es wurde noch keine Koordinate ermittelt!");        		
        		return;
        	}
        	if (!hasValidPosition) {
        		alert('Es liegt derzeit kein GPS-Signal vor');
        		return;
        	}
        	var position = lastPosition;
            navigator.camera.getPicture(function cameraSuccess(imageURI){
                var pos = {};
                pos["timestamp"] = position.timestamp;
                pos["longitude"] = position.coords.longitude;
                pos["latitude"] = position.coords.latitude;
                pos["accuracy"] = position.coords.accuracy;
                pos["altitude"] = position.coords.altitude;
                pos["altitudeAccuracy"] = position.coords.altitudeAccuracy;
                pos["heading"] = position.coords.heading;
                pos["speed"] = position.coords.speed;
                var posJsonObj = JSON.stringify(pos);
                imageCount += 1;
                app.db.executeSql("INSERT INTO image (trip_id, posJsonObj, created, uri) VALUES (?, ?, ?, ?)", [app.data.currentTrip, posJsonObj, new Date(), imageURI]);
                app.utils.debug.log(app.utils.debug.debugPages.JOB_TRACKER, imageURI);
            }, function cameraError(error){
                app.utils.debug.log(app.utils.debug.debugPages.JOB_TRACKER, "Unable to obtain picture", "ERROR");
                app.utils.debug.log(app.utils.debug.debugPages.JOB_TRACKER, error, "ERROR");
                alert("Bild nicht gespeichert!");
            }, cameraOptions);
        }

        function addNote(text){
        	if (lastPosition == null) {
        		alert("Es wurde noch keine Koordinate ermittelt!");        		
        		return;
        	}
        	if (!hasValidPosition) {
        		alert('Es liegt derzeit kein GPS-Signal vor');
        		return;
        	}
            app.db.executeSql("INSERT INTO note (trip_id, note, posJsonObj, created) VALUES (?, ?, ?, ?)", [app.data.currentTrip, text, lastPosition, new Date()], function(result){
                console.log(result);
            });
            $('#job-tracker-bottomsheet-note').panel().panel('close');
            $('#job-tracker-bottomsheet-note-textarea').val("");
            noteCount += 1;
        }


        function showLoadingAnimation(){
            loaderRef.fadeIn(500);
        }

        function hideLoadingAnimation(){
            loaderRef.fadeOut(500);
        }

        function updateLayout(){
            currentDistanceRef.text(app.utils.round(currentDistance, 2)+" m");
            var time = app.utils.timeIntervalToString(trackBegin, false);
            currentTimeRef.text(time);
            if(imageCount == 1){
                imageCounterRef.html("<span>1</span>Bild");
            } else {
                imageCounterRef.html("<span>"+imageCount+"</span>Bilder");
            }
            if(noteCount == 1){
                noteCounterRef.html("<span>1</span>Notiz");
            } else {
                noteCounterRef.html("<span>"+noteCount+"</span>Notizen");
            }
        }

        //public
        return {
        	start : function() {
        		startTracking();
        	},
            getIsTracking: function(){
                return isTracking;
            },
            beforeShow: function(){

            },
            beforeHide: function(){
                reset();
            },
            confirmEndTrip: function() {
            	confirmEndTrip();
            },
            showProtocol: function(show) {
            	showProtocol = show;
            }
        }
    }

    return {
        getInstance: function(){
            if(!instance){
                instance = init();
            }
            return instance;
        }
    }
})();
