/**
 * Created by christiankalig on 05.07.16.
 */

var app = app || {};

var Jobs = (function(){
    var instance;

    function init(){
        // private
        var jobsListRef = $('#jobs-list'),
            pageRefString = "#jobs";
        var url = localStorage.getItem("base_url");

        function loadJobs(){
            // insert oder update
            jobsListRef.empty();
            $.mobile.changePage(pageRefString);
            $.mobile.loading("show", {
                text: "Aufträge werden geladen",
                textVisible: true
            });
            app.db.getLoginData(function(username, authkey){
                $.ajax({
                    method: "GET",
                    url: url + "api/jobs/get",
                    data: {
                        user: username,
                        token: authkey
                    },
                    dataType: 'json',
                    xhrFields: {
                        withCredentials: true
                    },
                    success: function(result){
                    	if(result.success && result.data.jobs){
                            onJobsLoadedFromServer(result.data.jobs);
                        }
                    },
                    error: function(e){
                        app.utils.debug.log(app.utils.debug.debugPages.JOBS, e, "ERROR");
                    }
                });

            });



        }/// loadJobs  ---- END


        function onJobsLoadedFromServer(jobs) {
        	//get external ids
        	var ids = "";
            for(var i = 0; i < jobs.length; i++) {
            	ids += (ids != "" ? "," : "") + jobs[i].id;
            }
            //delete non existing
            app.db.executeSql("DELETE FROM job where external_id NOT IN (?)", [ids], false);
            app.db.executeSql("SELECT * FROM job", [], function(result){
            	var existing = new Array();
            	if (result.rows.length > 0) {
            		for(var i = 0; i < result.rows.length; i++) {
            			existing.push(result.rows.item(i).external_id);
            		}
            	}
            	//check for insert/update
            	for(var i = 0; i < jobs.length; i++) {
            		if ($.inArray(jobs[i]["id"], existing) == -1) {
            			app.db.executeSql(
                            "INSERT INTO job" +
                            "(description, address, creation_date, status, customer_name, is_active, external_id) " +
                            "VALUES (?, ?, ?, ?, ?, ?, ?)",
                            [jobs[i]["description"], jobs[i]["city"]+" "+jobs[i]["street"], jobs[i]["creationDate"], jobs[i]["status"], jobs[i]["jobcustomer"], 1, jobs[i]["id"]],
                            function(result){
                            }
                        );
            			existing.push(jobs[i]["id"]);
            		}
            		else {
            			app.utils.debug.log(app.utils.debug.debugPages.JOBS, jobs[i]["id"] + " existing");
            		}
            	}
            	app.db.executeSql("SELECT id, address, customer_name, external_id FROM job", [], onJobsLoaded);
            });
        }


        // get jobs from db
        function onJobsLoaded(result){
            app.utils.debug.log(app.utils.debug.debugPages.JOBS, "onJobsLoaded called");
            jobsListRef.append('<li data-role="list-divider">Offene Auftr&auml;ge</li>');
            var item;
            for(var i = 0; i < result.rows.length; i++){
                item = result.rows.item(i);
                jobsListRef.append(
                    '<li class="ui-li-has-thumb">' +
                    '   <a href="#trip-overview" class="job-item" data-id="'+item.id+'" data-extid="'+item.external_id+'">' +
                    '       <span class="ui-thumbnail ui-thumbnail-circular custom-thumbnail">#'+item.id+'</span>' +
                    '       <h2>'+item.address+'</h2>' +
                    '       <p>'+item.customer_name+'</p>' +
                    '   </a>' +
                    '</li>'
                );
            }
            /**
             * the jQuery Mobile listview widget needs to be refreshed after appending to it
             * (otherwise the jqm-css won't be applied to the list items)
             */
            jobsListRef.listview().listview('refresh');

            var jobsRef = $('.job-item');
            /**
             * bind click events to every list item
             */
            jobsRef.click(function(event, ui){
                app.utils.debug.log(app.utils.debug.debugPages.JOBS, "jobs ref click");
                app.data.currentJob = $(this).data('id');
                app.data.currentJobExtId = $(this).data('extid');
                app.pages.tripOverview.loadTripData();
            });
            /**
             * when all the work is done, show the page
             */
            $.mobile.loading("hide");
        }

        //public
        return {
            var_public: "public",
            onCreate: function(){
                loadJobs();
            },
            onJobsLoaded: function(result){
                onJobsLoaded(result);
            },
            show: function(){
                loadJobs();
            }
        }
    }

    return {
        getInstance: function(){
            if(!instance){
                instance = init();
            }
            return instance;
        }
    }
})();
