/**
 * Created by christiankalig on 05.07.16.
 */
var app = app || {};

var Protocol = (function(){
    var instance;

    function init(){

        String.prototype.format = function () {
            var args = arguments;
            return this.replace(/\{(\d+)\}/g, function (m, n) {
                return args[n];
            });
        };

        // private
        console.log("protocol loaded");
        var textarea = $('#note-for-image-textarea'),
            msg = $('#note-for-image-msg'),
            baseUrl;
        baseUrl = localStorage.getItem("base_url");

        function loadProtocolProfile() {
            app.db.getLoginData(function(username, authkey){
                $.ajax({
                    method: "POST",
                    url: baseUrl+"api/protocolProfile/find",
                    data: {
                        user: username,
                        token: authkey,
                        data: {
                            id: app.data.currentJobExtId
                        }
                    },
                    dataType: "json",
                    success: function(result){
                        console.log("PROTOKOLL-PROFIL result", result);
                        var protocol = JSON.parse(result.data.profile.profile);
                        console.log("PARSED PROFILE", protocol);
                        showProtocol(protocol);
                    },
                    error: function(e){
                        console.log("PROTOKOLL-PROFIL error", e);
                    }
                });

            });
        }

        function showProtocol(protocol) {
            var set;
            for(var i = 0; i < protocol.groups.length; i++) {
                set = showGroup(protocol.groups[i]);
            }
            set.collapsibleset("refresh");
        }

        String.prototype.hashCode = function() {
            var hash = 0, i, chr;
            if (this.length === 0) return hash;
            for (i = 0; i < this.length; i++) {
                chr   = this.charCodeAt(i);
                hash  = ((hash << 5) - hash) + chr;
                hash |= 0; // Convert to 32bit integer
            }
            return hash;
        };

        function showGroup(group) {
            var set = $('#protocol-collapsibleset');
            var id = "pc-" + group.name;
            var content = "<div data-role='collapsible' id='"+id+"'><h3>"+ group.prettyName +"</h3><div id='flip-"+id+"'></div></div>";
            set.append(content);
            addFields(group.fields, id, group);
            return set;
        }

        function addFields(fields, id, group) {
        	if (fields.length > 0) {
	            var content = $('#collapsible-headline').html();
	            for(var i = 0; i < fields.length; i++) {
	                var field = fields[i];
	                if(!field.visible) continue;
	                var fieldId = (field.name + id + "").hashCode();
	                content += $('#switchTemplate').html().format(field.name, fieldId, group.name, group.prettyName);
	            }
	            $('#flip-' + id).html(content).enhanceWithin();
	            $('#' + id).show();
        	}
        	else {
        		$('#' + id).hide();
        	}
        }

        function refreshMsg(){
            var countTrue = $('#note-protocol .flippy-flip option[value="true"]:selected').length,
                count = $('#note-protocol .flippy-flip').length,
                msg = $('#note-for-image-msg');
            console.log(countTrue, count);
            if(countTrue > 0){
                msg.removeClass("success").addClass("fail");
                msg.empty();
                msg.append('<h6><i class="zmdi zmdi-close-circle"></i> <i>'+countTrue+' von '+count+' beanstandet</i></h6>');
            } else {
                msg.removeClass("fail").addClass("success");
                msg.empty();
                msg.append('<h6><i class="zmdi zmdi-check-circle"></i> <i>Keine Beanstandungen</i></h6>');
            }
        }

        function onBeforeShow(isTracking){
            console.log("Protocol: before-show isTracking: "+isTracking);
            if(isTracking){
                $('#note-protocol-back-btn').show();
                $('#note-protocol-save-btn').hide();
            } else {
                $('#note-protocol-back-btn').hide();
                $('#note-protocol-save-btn').show();
            }
        }

        function resetProtocol(){
            $('#note-protocol select').each(function(index){
                $(this).val("false");
                $(this).flipswitch("refresh");
                for(var i = 1; i <= 6; i++){
                    refreshMsg(i);
                }
            });
        }

        function collectData() {
            var sortedData = {};
            var data = [];
            $('.flip-row').each(function() {
                // data sorted by groups and with more information
                var temp = {};
                temp.label = $(this).find("label").text();
                temp.value = $(this).find("select").val() === 'true' ? "NOK" + temp.label : "OK" + temp.label;
                temp.groupName = $(this).data("group");
                temp.groupPrettyName = $(this).find("input:first-child").val();
                if(temp.label.charAt(0) === '{') return true;
                if(temp.groupName in data) {
                    sortedData[temp.groupName].push(temp);
                } else {
                    sortedData[temp.groupName] = [];
                    sortedData[temp.groupName].push(temp);
                }
                // data for using the same technique as before
                var temp_1 = {};
                temp_1.jobId = app.data.currentJobExtId;
                temp_1.tripId = app.data.currentTripExtId;
                temp_1.lat = 0;
                temp_1.lon = 0;
                temp_1.key = temp.groupPrettyName;
                temp_1.value = temp.value;
                data.push(temp_1);
            });
            data.push({jobId: app.data.currentJobExtId, tripId: app.data.currentTripExtId, lat: 0, lon: 0, key: "TEXTAREA", value: msg.val()});
            console.log("COLLECT DATA", data, sortedData);
            return {data: data, sortedData: sortedData};
        }

        function saveProtocol(){
        	if (app.data.currentTripExtId == null || app.data.currentTripExtId <= 0) {
        		alert("Aktueller Trip ist nicht bekannt! Protokoll kann nicht gespeichert werden.");
        		return;
        	}
            app.db.executeSql("INSERT INTO note (trip_id, note, posJsonObj, created) VALUES (?, ?, ?, ?)", [app.data.currentTrip, textarea.val(), null, new Date()], function(result){
                console.log("TEXTAREA NOTE ADDED");
                console.log(result);
            });
            var data = collectData().data;
            app.db.getLoginData(function(username, authkey){
                $.ajax({
                    method: "POST",
                    url: baseUrl+"api/note/addmultiple",
                    data: {
                        user: username,
                        token: authkey,
                        data: data
                    },
                    dataType: "json",
                    success: function(result){
                        console.log(result);
                        app.db.executeSql("INSERT INTO trip_ended (trip_id, job_id) VALUES (?, ?)", [app.data.currentTrip, app.data.currentJob], false,
                                function(error){alert(error);}
                            );
                        resetProtocol();
                        app.data.isTracking = false;
                    },
                    error: function(e){
                        console.log(e);
                    }
                });

            });
        }

        //public
        return {
            var_public: "public",
            onSelectChangeInsideTab: function(){
                console.log("FLIP CHANGED");
                refreshMsg();
            },
            beforeShow: function(isTracking){
                onBeforeShow(isTracking);
                loadProtocolProfile();
            },
            onSaveProtocolAction: function(){
                //collectData();
                saveProtocol();
            }
        }
    }

    return {
        getInstance: function(){
            if(!instance){
                instance = init();
            }
            return instance;
        }
    }
})();
