/**
 * Created by christiankalig on 05.07.16.
 */
var app = app || {};

var TripOverview = (function(){
    var instance;
    var rememberedToken = "";
    var rememberedUser = "";

    function init(){
        // private
        var newTripBtn = $('#trip-overview-new-trip'),
            tripOverviewList = $('#trip-overview-list'),
            anordnungDocumentsList = $('#anordnung-documents-list'),
            docsAnordnungDocumentsList = $('#docs-anordnung-documents-list'),
            regelplanDocumentsList = $('#regelplan-documents-list'),
            docsRegelplanDocumentsList = $('#docs-regelplan-documents-list'),
            documentsList = $('#job-documents-list'),
            anordnungDiv = $('#anordnungDiv'),
            regelplaneDiv = $('#regelplaneDiv'),
            sonstigeDiv = $('#sonstigeDiv');

        newTripBtn.click(function(event){
            console.log("new trip btn clicked");
            app.pages.jobOverview.loadJobData(app.data.currentJob);
        });

        /**
         * loads a trip by currently opened job
         */
        function loadTripData(){
        	resetMetaData();
        	resetDocuments();
            app.db.executeSql("SELECT t.*, j.customer_name FROM trip t, job j WHERE t.job_id = ? AND t.job_id = j.id", [app.data.currentJob], onTripDataLoaded);
            app.db.getLoginData(function(username, authkey){
            	if(authkey && username) {
            		rememberedToken = authkey;
            		rememberedUser = username;
                    //load meta data
            		$.ajax({
                        method: 'GET',
                        url: localStorage.getItem("base_url")+"api/jobs/find",
                        data: {
                            token: authkey,
                            user: username,
                            id: app.data.currentJobExtId
                        },
                        dataType: 'json',
                        success: function(result){
                            console.log(result);
                            if(result.success && result.data && result.data.job){
                                onMetaDataLoaded(result.data.job);
                            } else {
                                console.log("Meta data not loaded");
                            }
                        },
                        error: function(e){
                            app.utils.debug.log(app.utils.debug.debugPages.TRIP_OVERVIEW, e);
                        }
                    });
            		//load documents
            		$.ajax({
                        method: 'GET',
                        url: localStorage.getItem("base_url")+"api/doc/getall",
                        data: {
                            token: authkey,
                            user: username,
                            jobId: app.data.currentJobExtId
                        },
                        dataType: 'json',
                        success: function(result){
                            app.utils.debug.log(app.utils.debug.debugPages.TRIP_OVERVIEW, result);
                            app.utils.debug.log(app.utils.debug.debugPages.TRIP_OVERVIEW, result.data);
                            if(result.success && result.data && result.data.docs){
                                onDocumentsLoaded(result.data.docs);
                            } else {
                                app.utils.debug.log(app.utils.debug.debugPages.TRIP_OVERVIEW, "Documents not loaded");
                            }
                        },
                        error: function(e){
                            app.utils.debug.log(app.utils.debug.debugPages.TRIP_OVERVIEW, e);
                        }
                    });
                } else {
                    app.utils.debug.log(app.utils.debug.debugPages.TRIP_OVERVIEW, "Login data not loaded ");
                    return false;
                }
            });
        }

        /**
         * resets displayed documents
         */
        function resetDocuments() {
            documentsList.empty();
            anordnungDocumentsList.empty();
            regelplanDocumentsList.empty();
            docsAnordnungDocumentsList.empty();
            docsRegelplanDocumentsList.empty();
            
            docsAnordnungDocumentsList.append("Keine Dokumente verfügbar!");
            docsRegelplanDocumentsList.append("Keine Dokumente verfügbar!");
            
            anordnungDiv.css('display', 'none');
            regelplaneDiv.css('display', 'none');
            sonstigeDiv.css('display', 'none');
        }

        /**
         * formats a number
         */
        function formatThousandsWithRounding(n, dp){
        	var w = (typeof(n.toFixed) == "function" ? n.toFixed(dp) : 0), k = w|0, b = n < 0 ? 1 : 0,
              u = Math.abs(w-k), d = (''+u.toFixed(dp)).substr(2, dp),
              s = ''+k, i = s.length, r = '';
        	while ( (i-=3) > b ) { r = ',' + s.substr(i, 3) + r; }
        	return s.substr(0, i + 3) + r + (d ? '.'+d: '');
        }

        /**
         * formats a document size
         */
        function formatSize(size) {
            if (size > 1024) {
            	size = size / 1024;
            	if (size > 1024) {
            		size = size / 1024;
            		if (size > 1024) {
            			size = size / 1024;
            			size = formatThousandsWithRounding(size, 2) + " gb";
            		}
            		else {
            			size = formatThousandsWithRounding(size, 2) + " mb";
            		}
            	}
            	else {
            		size = formatThousandsWithRounding(size, 2) + " kb";
            	}
            }
            else {
            	size = formatThousandsWithRounding(size, 2) + " b";
            }
            return size;
        }

        /**
         * displays loaded documents by job
         */
        function onDocumentsLoaded(docs) {
        	//display docs
            resetDocuments();
            var item;
            var docsAnordnungFirst = true;
            var docsRegelplanFirst = true;
            for(var i = 0; i < docs.length; i++){
                item = docs[i];
                var stringToApend = "<li class='ui-li-has-thumb'>" +
                        "   <a href='#' class='doc-item' data-id='"+item.id+"'>" +
                        "       <span class='ui-thumbnail ui-thumbnail-circular custom-thumbnail'>" +
                        "#"+item.id +
                        "</span>" +
                        "       <h2>" +
                        item.documentName +
                        "       </h2>" +
                        "       <p>" +
                        item.filename +
                        "       </p>" +
                        "       <p>" +
                        size +
                        "       </p>" +
                        "   </a>" +
                        "</li>";
                var size = formatSize(item.size);
                console.log("STRING TO APPEND", stringToApend);
                 if (item.doccategory == 'Verkehrsrechtliche Anordnung') {
                    anordnungDiv.css('display', 'block');
                    anordnungDocumentsList.append(stringToApend);
                    docsAnordnungDocumentsList.append(stringToApend);
                 }

                 else if (item.doccategory == 'Regelpläne') {
                    regelplaneDiv.css('display', 'block');
                    regelplanDocumentsList.append(stringToApend);
                    docsRegelplanDocumentsList.append(stringToApend);

                 } else {
                    sonstigeDiv.css('display', 'block');
                    documentsList.append(stringToApend);
                }
            }//for end

            documentsList.listview().listview('refresh');
            anordnungDocumentsList.listview().listview('refresh');
            docsAnordnungDocumentsList.listview().listview('refresh');
            regelplanDocumentsList.listview().listview('refresh');
            docsRegelplanDocumentsList.listview().listview('refresh');
            $('.doc-item').dblclick(function(event){
                app.utils.debug.log(app.utils.debug.debugPages.TRIP_OVERVIEW, "download " + $(this).data('id'));
                var base_url = localStorage.getItem("base_url");
                var documentId = $(this).data('id');
                $.ajax({
                    method: 'GET',
                    url: base_url+"api/doc/get",
                    data: {
                        token: rememberedToken,
                        user: rememberedUser,
                        jobId: app.data.currentJobExtId,
                        docId: documentId
                    },
                    dataType: 'json',
                    success: function(result){
                        app.utils.debug.log(app.utils.debug.debugPages.TRIP_OVERVIEW, result);
                        if(result.success && result.data && result.data.doc) {
                        	var filename = app.data.currentJob + "_" + documentId + "_" + result.data.doc.filename;
                        	var mimetype = result.data.doc.mimetype;
                        	//file already existis?
                        	window.resolveLocalFileSystemURL(cordova.file.externalApplicationStorageDirectory, function (directoryEntry) {
                        		directoryEntry.getFile(filename, { create: false, exclusive: false }, function (fileEntry) {
                        			app.utils.debug.log(app.utils.debug.debugPages.TRIP_OVERVIEW, "open file");
                        			openFile(filename, mimetype);
                        		}, function () {
                        			new $.nd2Toast({ // The 'new' keyword is important, otherwise you would overwrite the current toast instance
                             		   message : "Lade " + formatSize(result.data.doc.size) + " herunter", // Required
                             		   ttl : 2000 // optional, time-to-live in ms (default: 3000)
                             		 });
	                             	$.ajax({
	                                     dataType: 'native',
	                                     method: 'POST',
	                                     url: base_url+"api/doc/download",
	                                     data: {
	                                         token: rememberedToken,
	                                         user: rememberedUser,
	                                         jobId: app.data.currentJobExtId,
	                                         docId: documentId
	                                     },
	                                     xhrFields: {
	                                     	responseType: 'arraybuffer'
	                                     },
	                                     success: function(resp) {
	                                     	app.utils.debug.log(app.utils.debug.debugPages.TRIP_OVERVIEW, resp);
	                                     	saveData(resp, filename, mimetype);
	                                     },
	                                     error: function(e){
	                                    	 alert("Datei konnte nicht heruntergeladen werden.");
	                                         app.utils.debug.log(app.utils.debug.debugPages.TRIP_OVERVIEW, e);
	                                     }
	                                 });
                        		});
                        	});
                        } else {
                            app.utils.debug.log(app.utils.debug.debugPages.TRIP_OVERVIEW, "Document not loaded");
                        }
                    },
                    error: function(e){
                        app.utils.debug.log(app.utils.debug.debugPages.TRIP_OVERVIEW, e);
                    }
                });
            });
        }

        /**
         * openes a file
         */
        function openFile(fileName, mimeType) {
        	cordova.plugins.fileOpener2.open(cordova.file.externalApplicationStorageDirectory + fileName, mimeType,
        			{
                        error: function (e) {
                            app.utils.debug.log(app.utils.debug.debugPages.TRIP_OVERVIEW, 'Error status: ' + e.status + ' - Error message: ' + e.message);
                        },
                        success: function () {
                            app.utils.debug.log(app.utils.debug.debugPages.TRIP_OVERVIEW, 'file opened successfully');
                        }
                    }
               );
        }

        /**
         * saves a blog
         */
        function saveData(data, fileName, mimeType) {
            app.utils.debug.log(app.utils.debug.debugPages.TRIP_OVERVIEW, fileName);
            try {
                window.resolveLocalFileSystemURL(cordova.file.externalApplicationStorageDirectory, function (directoryEntry) {
                    directoryEntry.getFile(fileName, { create: true, exclusive: false }, function (fileEntry) {
                        fileEntry.createWriter(function (fileWriter) {
                        	app.utils.debug.log(app.utils.debug.debugPages.TRIP_OVERVIEW, "file created");
                            fileWriter.onwriteend = function (e) {
                            	openFile(fileName, mimeType);
                            };
                            fileWriter.onerror = function (e) {
                                alert(e);
                            };
                            var blob = new Blob([data], { type: mimeType });
                            app.utils.debug.log(app.utils.debug.debugPages.TRIP_OVERVIEW, "write file " + mimeType);
                            fileWriter.write(blob);
                        }, function onerror(e) {
                            alert(e);
                        });
                    }, function onerror(e) {
                        alert(e);
                    });
                }, function onerror(e) {
                    alert(e);
                });
            } catch (e) {
                 alert(e);
            }
        }

        /**
         * resets jobs meta data
         */
        function resetMetaData() {
        	$('#job-meta-customer').html("");
        	$('#job-meta-assignedto').html("");
        	$('#job-meta-status').html("");
        	$('#job-meta-description').html("");
        	$('#job-meta-city').html("");
        	$('#job-meta-area').html("");
        	$('#job-meta-worktype').html("");
        	$('#job-meta-responsible').html("");
        	$('#job-meta-phone1').html("");
        	$('#job-meta-phone1').attr('href', "");
        	$('#job-meta-phone2').html("");
        	$('#job-meta-phone2').attr('href', "");
        	$('#workplace-select').html("");
        	$('#job-overview-client').html("");
            $('#emails-select').html("");
        }

        /**
         * displays jobs meta data
         */
        function onMetaDataLoaded(job) {
        	//display meta data
        	$('#job-meta-customer').html(job['jobcustomer']);
        	$('#job-meta-assignedto').html(job['userArranger']);
        	$('#job-meta-status').html(job['status']);
        	$('#job-meta-description').html(job['description']);
        	$('#job-meta-city').html(job['city']+" "+job['street']);
        	$('#job-meta-area').html(job['work']);
        	$('#job-meta-worktype').html(job['type']);
        	$('#job-meta-responsible').html(job['responsible']);
        	$('#job-meta-phone1').html(job['phone']);
        	$('#job-meta-phone1').attr('href', "tel:"+job['phone']);
        	$('#job-meta-phone2').html(job['phone2']);
        	$('#job-meta-phone2').attr('href', "tel:"+job['phone2']);
        	$('#workplace-select').html(job['work']);
        	$('#job-overview-client').html(job['jobcustomer']);
            $('#emails-select').html(job['emails']);
        }

        /**
         * displays loaded trips
         */
        function onTripDataLoaded(result){
            tripOverviewList.empty();
            var item;
            for(var i = 0; i < result.rows.length; i++){
                item = result.rows.item(i);
                tripOverviewList.append(
                    "<li class='ui-li-has-thumb'>" +
                    "   <a href='#' class='trip-item' data-id='"+item.id+"'>" +
                    "       <span class='ui-thumbnail ui-thumbnail-circular custom-thumbnail'>" +
                    "#"+item.id +
                    "</span>" +
                    "       <h2>" +
                    item.customer_name +
                    "       </h2>" +
                    "       <p>" +
                    item.start +
                    "       </p>" +
                    "   </a>" +
                    "</li>"
                );
            }
            tripOverviewList.listview().listview('refresh');
            $('.trip-item').click(function(event){
                app.data.currentTrip = $(this).data('id');
                app.utils.debug.log(app.utils.debug.debugPages.TRIP_OVERVIEW, "current trip="+app.data.currentTrip);
                if (app.data.currentTrip > 0) {
                	app.pages.jobReview.showMap();
                }
            });
        }

        //public
        return {
            beforeShow: function(){
                loadTripData();
            },
            loadTripData: function(){
                loadTripData();
            }
        }
    }

    return {
        getInstance: function(){
            if(!instance){
                instance = init();
            }
            return instance;
        }
    }
})();
