/**
 * Created by christiankalig on 11.05.16.
 */

var app = app || {};

var Database = (function(){
    var instance;

    function init(){
        console.log("Database init");
        // private
        var settings = {
            name: "RSA_DB_1",
            location: "default"
        };

        function initDatabase() {
        	//dropDatabaseTables();
            //get app version number
            var versionnumber = "";
            createDatabaseTables();
            cordova.getAppVersion.getVersionNumber().then(function (versionnumber) {
            	$('#app-version').html(versionnumber);
            	if (versionnumber == null || versionnumber == "" || versionnumber == "0.7.7") {
                	dropDatabaseTables();
                }
                else {
                	$('#app-version').html(versionnumber);
                }
                //set base url
                app.db.setMeta("base_url", "http://ztv-sa.4e-software.de/");
                //try getting version number from database
                app.db.getMeta("app_version", function(version) {
                	if (version == null || version == "" || version == "0.7.8") {
                		console.log("refresh db!");
                		//refresh tables
                		dropDatabaseTables();
                		createDatabaseTables();
                		//set base url
                        app.db.setMeta("base_url", "http://ztv-sa.4e-software.de/");
                		app.utils.debug.log(app.utils.debug.debugPages.DB, "no version added to meta yet");
                       	updateDatabase(versionnumber, "0.0.0");
                	}
                	else {
    	            	app.utils.debug.log(app.utils.debug.debugPages.DB, "Known version: " + version);
    	            	updateDatabase(versionnumber, version);
                	}
                	app.utils.debug.log(app.utils.debug.debugPages.DB, "init database finished");
                    if (app.pages.login != null) {
                    	app.pages.login.onShow();
                    }
                    else {
                    	app.utils.debug.log(app.utils.debug.debugPages.DB, "login not initialized yet");
                    }
                });
            });

        }

        /**
         * deletes all database tables
         */
        function dropDatabaseTables() {
        	database.executeSql("DROP TABLE IF EXISTS trip");
            database.executeSql("DROP TABLE IF EXISTS job");
            database.executeSql("DROP TABLE IF EXISTS note");
            database.executeSql("DROP TABLE IF EXISTS position");
            database.executeSql("DROP TABLE IF EXISTS image");
            database.executeSql("DROP TABLE IF EXISTS meta");
            database.executeSql("DROP TABLE IF EXISTS trip_ended");
        }

        /**
         * creates database tables if not existing yet
         */
        function createDatabaseTables() {
        	database.executeSql(
                    "CREATE TABLE IF NOT EXISTS job (" +
                    "id INTEGER PRIMARY KEY," +
                    "external_id INTEGER," +
                    "description VARCHAR," +
                    "address VARCHAR," +
                    "creation_date DATETIME," +
                    "status VARCHAR," +
                    "customer_name VARCHAR," +
                    "is_active BIT" +
                    ")",
                    [], false
                );

                database.executeSql(
                    "CREATE TABLE IF NOT EXISTS trip (" +
                    "id INTEGER PRIMARY KEY," +
                    "job_id INTEGER NOT NULL," +
                    "external_id INTEGER NOT NULL DEFAULT -1," +
                    "start DATETIME," +
                    "end DATETIME," +
                    "user_id INTEGER NOT NULL," +
                    "note VARCHAR," +
                    "is_active BIT NOT NULL DEFAULT '1'" +
                    ")",
                    [], false
                );

                database.executeSql(
                    "CREATE TABLE IF NOT EXISTS image (" +
                    "id INTEGER PRIMARY KEY," +
                    "trip_id INTEGER," +
                    "posJsonObj VARCHAR," +
                    "uri VARCHAR," +
                    "created VARCHAR," +
                    "synced BIT DEFAULT 0" +
                    ")",
                    [], false
                );

                database.executeSql(
                    "CREATE TABLE IF NOT EXISTS note (" +
                    "id INTEGER PRIMARY KEY," +
                    "trip_id INTEGER," +
                    "note VARCHAR," +
                    "posJsonObj VARCHAR," +
                    "created VARCHAR," +
                    "synced BIT DEFAULT 0" +
                    ")",
                    [], false
                );

                database.executeSql(
                    "CREATE TABLE IF NOT EXISTS position (" +
                    "id INTEGER PRIMARY KEY," +
                    "trip_id INTEGER NOT NULL," +
                    "posJsonObj VARCHAR," +
                    "synced BIT DEFAULT 0" +
                    ")",
                    [], false
                );

                database.executeSql(
                    "CREATE TABLE IF NOT EXISTS meta (" +
                    "id INTEGER PRIMARY KEY," +
                    "meta_key VARCHAR," +
                    "meta_value VARCHAR" +
                    ")",
                    [], false
                );

                app.db.executeSql(
                    "CREATE TABLE IF NOT EXISTS trip_ended (" +
                    "id INTEGER PRIMARY KEY," +
                    "job_id INTEGER," +
                    "trip_id INTEGER" +
                    ")", [], false
                );

                app.db.executeSql(
                    "IF DOES NOT EXIST ALTER TABLE trip ADD external_id INTEGER NULL DEFAULT -1",
                    [], false
                );
        }

        /**
         * called for updating the database depending on the version number
         */
        function updateDatabase(newVersion, oldVersion) {
        	if (newVersion == oldVersion) {
        		//nothing to do
        		return;
        	}
        	var currentArr = newVersion.split(".");
        	var oldArr = oldVersion.split(".");
        	if (currentArr.length > 2 && oldVersion.length > 2) {
        		//version 0.7.2 with new database fields. Old version isn't important due to the "if does not exist" statement
        		if (currentArr[0] >= 0 && currentArr[1] >= 7 && currentArr[2] >= 2) {
        			app.utils.debug.log(app.utils.debug.debugPages.DB, "upgrade from " + oldVersion + " to " + newVersion);
        			//nothing to do
        		}
        	}
        	else {
        		//nothing to do
        		return;
        	}
        	app.db.setMeta("app_version", newVersion);
        }

        function onDbError(){
            app.utils.debug.log(app.utils.debug.debugPages.DB, "error db");
        }

        var database = window.sqlitePlugin.openDatabase(settings, initDatabase, onDbError);

        //public
        return {
            executeSql: function(query, args, cb, errorCb){
                database.executeSql(query, args, cb, errorCb);
            },
            setMeta: function(key, value){
                database.executeSql("SELECT * FROM meta WHERE meta_key = ?", [key], function(result){
                    if(result.rows.length == 0){
                        database.executeSql("INSERT INTO meta (meta_key, meta_value) VALUES (?, ?)", [key, value], function(result){
                        	app.utils.debug.log(app.utils.debug.debugPages.DB, "META KEY "+key+" MIT WERT "+value+" IN META-TABELLE EINGETRAGEN");
                        });
                    } else {
                        database.executeSql("UPDATE meta SET meta_value = ? WHERE meta_key = ?", [value, key], function(result){
                        	app.utils.debug.log(app.utils.debug.debugPages.DB, "META KEY "+key+" MIT WERT "+value+" IN META-TABELLE EINGETRAGEN");
                        });
                    }
                });
            },
            getMeta: function(key, cb){
                database.executeSql("SELECT * FROM meta WHERE meta_key = ?", [key], function(result){
                    if(result.rows.length == 0){
                        cb(null);
                        return false;
                    }
                    else {
	                    var item = result.rows.item(0);
	                    app.utils.debug.log(app.utils.debug.debugPages.DB, item);
	                    cb(item.meta_value);
                    }
                });
            },
            getLoginData: function(cb){
                database.executeSql("SELECT * FROM meta WHERE meta_key = 'user_loginname' or meta_key = 'user_authkey'", [], function(result){
                    try {
	                	if(result.rows.length != 2){
	                        return false;
	                    }
	                    var username = false;
	                    var authkey = false;
	                    if (result.rows.item(0).meta_key == "user_loginname") {
	                    	username = result.rows.item(0).meta_value;
	                    	authkey = result.rows.item(1).meta_value;
	                    }
	                    else {
	                    	username = result.rows.item(1).meta_value;
	                    	authkey = result.rows.item(0).meta_value;
	                    }
	                    cb(username, authkey);
                    }
                    catch (err) {
                    	app.utils.debug.log(app.utils.debug.debugPages.DB, "getLoginData - error!");
                    	app.utils.debug.log(app.utils.debug.debugPages.DB, err);
                    }
                });
            },
            getExternalTripIdByInternalTripId: function(id, cb){
                database.executeSql("SELECT external_id FROM trip WHERE id = ?", [id], function(result){
                    if(result.rows.length == 0){
                        cb(-1);
                    } else {
                        cb(result.rows.item(0).external_id);
                    }
                });
            },
            getExternalJobIdByInternalJobId: function(id, cb){
                database.executeSql("SELECT external_id FROM job WHERE id = ?", [id], function(result){
                    if(result.rows.length == 0){
                        cb(-1);
                    } else {
                        cb(result.rows.item(0).external_id);
                    }
                });
            },
            getExternalJobIdByInternalJobIdAndGetExternalTripIdByInternalTripId: function(job_id, trip_id, cb){
                database.executeSql("SELECT j.external_id AS ext_job_id, t.external_id AS ext_trip_id FROM job j, trip t WHERE j.id = ? AND t.id = ?", [job_id, trip_id], function(result){
                    if(result.rows.length == 0){
                    	alert('Keine Fahrt-ID und Auftrag-Id fr Trip ' + trip_id + " und Job " + job_id + " gefunden! Daten knnen nicht bermittelt werden.");
                        cb(-1);
                    } else {
                        var item = result.rows.item(0);
                        cb(item.ext_job_id, item.ext_trip_id);
                    }
                });
            },
            getExternalJobIdAndExternalTripIdByInternalTripId: function(trip_id, cb){
                database.executeSql("SELECT j.external_id AS ext_job_id, t.external_id AS ext_trip_id FROM job j, trip t WHERE t.id = ? AND t.job_id = j.id", [trip_id], function(result){
                    if(result.rows.length == 0){
                    	alert('Keine Fahrt-ID fr Trip ' + trip_id + " gefunden! Daten knnen nicht bermittelt werden.");
                        cb(-1);
                    } else {
                        var item = result.rows.item(0);
                        console.log("getExternalJobIdAndExternalTripIdByInternalTripId");
                        console.log(item);
                        cb(item.ext_job_id, item.ext_trip_id);
                    }
                }, function(error){
                    console.log(error);
                });
            }
        }
    }

    return {
        getInstance: function(){
            if(!instance){
                instance = init();
            }
            return instance;
        }
    }
})();
