/**
 * Created by christiankalig on 05.07.16.
 */
var app = app || {};

var Synchronisator = (function () {
    var instance;

    function init() {
        // private

        var baseUrl,
            lastSynchronisation,
            ADD_WAY_POINTS = "api/note/addwaypoints",
            ADD_IMAGE = "api/note/addimage",
            ADD_NOTE = "api/note/add",
            timer,
            IN_WAYPOINT_SYNC = false,
            IN_IMAGE_SYNC = false,
            IN_NOTE_SYNC = false,
            SYNC_ID = false,
            lastFinishedId = -1;
        baseUrl = localStorage.getItem("base_url");

        function startSyncing() {
            timer = window.setInterval(function () {

                app.utils.debug.log(app.utils.debug.debugPages.SYNCHRONISATOR, "SYNCING CALLED", "INFO", 0);

                if (app.data.isTracking) {
                    console.log("SYNC LIVE DATA");
                    if (!IN_WAYPOINT_SYNC) {
                        syncWaypoint(app.data.currentTrip);
                    }

                    if (!IN_IMAGE_SYNC) {
                        syncImage(app.data.currentTrip);
                    }

                    if (!IN_NOTE_SYNC) {
                        syncNote(app.data.currentTrip);
                    }
                } else {
                    console.log("SYNC OLD DATA");
                    app.db.executeSql("SELECT * FROM trip_ended WHERE trip_id IS NOT NULL LIMIT 1", [], function (trips) {
                        if (trips.rows.length == 0) {
                            console.log("trip_ended empty");
                            return;
                        }
                        var imagesLeft,
                            notesLeft,
                            waypointsLeft,
                            item = trips.rows.item(0);
                        app.db.executeSql("SELECT * FROM image WHERE synced = 0 AND trip_id = ?", [item.trip_id], function (result) {
                            imagesLeft = result.rows.length;
                            app.db.executeSql("SELECT * FROM note WHERE synced = 0 AND trip_id = ?", [item.trip_id], function (result1) {
                                notesLeft = result1.rows.length;
                                app.db.executeSql("SELECT * FROM position WHERE synced = 0 AND trip_id = ?", [item.trip_id], function (result2) {
                                    waypointsLeft = result2.rows.length;
                                    if (imagesLeft == 0 && notesLeft == 0 && waypointsLeft == 0) {
                                        //now we know that this trip is finished and all related data is transmitted -> end trip on server and remove trip from db
                                        app.db.getExternalJobIdByInternalJobIdAndGetExternalTripIdByInternalTripId(item.job_id, item.trip_id, function (ext_job_id, ext_trip_id) {
                                            app.db.getLoginData(function (username, authkey) {
                                                $.ajax({
                                                    method: "GET",
                                                    url: localStorage.getItem("base_url") + "api/trip/end",
                                                    data: {
                                                        user: username,
                                                        token: authkey,
                                                        jobId: ext_job_id,
                                                        tripId: ext_trip_id,
                                                        sendMail: true,
                                                        duration: new Date(),
                                                        distance: 0
                                                    },
                                                    dataType: 'json',
                                                    xhrFields: {
                                                        withCredentials: true
                                                    },
                                                    success: function (result) {
                                                        console.log(result);
                                                    	app.utils.debug.log(app.utils.debug.debugPages.JOB_TRACKER, "Trip beendet", "INFO", ext_trip_id);
                                                        console.log("TRIP " + item.trip_id + " WURDE BEENDET");
                                                        app.db.executeSql("DELETE FROM trip_ended WHERE trip_id = ?", [item.trip_id]);
                                                        if (lastFinishedId != ext_trip_id) {
                                                        	alert('Fahrt ' + ext_trip_id + ' beendet!');
                                                        	//avoid displaying the message multiple times
                                                        	lastFinishedId = ext_trip_id;
                                                        }
                                                    },
                                                    error: function (req, status, e) {
                                                    	console.error("fahrt " + ext_trip_id + " nicht beendet: " + req.responseText);
                                                        app.utils.debug.log(app.utils.debug.debugPages.JOB_TRACKER, req, "ERROR", ext_trip_id);
                                                        app.utils.debug.log(app.utils.debug.debugPages.JOB_TRACKER, e, "ERROR", ext_trip_id);
                                                        if (!req.responseText.startsWith("setup")) {
                                                        	alert('Fahrt ' + ext_trip_id + ' konnte nicht beendet werden: ' + req.responseText);
                                                        }
                                                    }
                                                });
                                            });
                                        });
                                    } else {
                                        //make sure that this data will synchronize
                                        console.log("TRIP " + item.trip_id + " HAT NOCH " + imagesLeft + " BILDER, " + notesLeft + " Notizen und " + waypointsLeft + " WEGPUNKTE");
                                        syncImage(item.trip_id);
                                        syncNote(item.trip_id);
                                        syncWaypoint(item.trip_id);
                                    }
                                });
                            });
                        });
                    }, function (error) {
                        console.log(error);
                    });
                }
            }, 5000);
        }

        function syncImage(id) {
            if (IN_IMAGE_SYNC) {
                return;
            }
            app.db.executeSql("SELECT * FROM image WHERE synced = 0 AND trip_id = ? ORDER BY id DESC LIMIT 1", [id], function (result) {
                if (result.rows.length == 0) {
                    console.log("no images for " + id);
                    return;
                }
                console.log(result.rows.length + " images");
                var item = result.rows.item(0);
                console.log(item);
                var fileURL = item.uri;
                var pos = $.parseJSON(item.posJsonObj);
                console.log("POS");
                console.log(pos);
                console.log(fileURL);
                app.db.getLoginData(function (username, authkey) {
                    if (IN_IMAGE_SYNC) {
                        return;
                    }
                    // Convert image
                    if (IN_IMAGE_SYNC) {
                        return;
                    }
                    // Then you'll be able to handle the myimage.png file as base64
                    var start = new Date();
                    app.db.getExternalJobIdAndExternalTripIdByInternalTripId(id, function (job_id, trip_id) {
                        if (IN_IMAGE_SYNC) {
                        	app.utils.debug.log(app.utils.debug.debugPages.SYNCHRONISATOR, 
                                	"Bildübertragung wird gerade durchgeführt...", "INFO", trip_id);
                            return;
                        }
                        IN_IMAGE_SYNC = true;
                        console.log("JOB_ID: " + job_id + " TRIP_ID: " + trip_id);
                        console.log("JETZT ITEM");
                        console.log(item);
                        var name;

                        var options = new FileUploadOptions();
                        options.fileKey = "file";
                        if (device.platform === "iOS") {
                            name = new Date().getTime();
                            var jpgTest = new RegExp("jpg"),
                                JPGTest = new RegExp("JPG"),
                                pngTest = new RegExp("png"),
                                PNGTest = new RegExp("PNG");
                            if (jpgTest.test(item.uri) || JPGTest.test(item.uri)) {
                                name += ".jpg";
                            } else if (pngTest.test(item.uri) || PNGTest.test(item.uri)) {
                                name += ".png";
                            }
                            options.fileName = name;
                        } else {
                            options.fileName = fileURL.substr(fileURL.lastIndexOf('/') + 1);
                        }
                        options.mimeType = "image/jpeg";

                        var params = {};
                        params.user = username;
                        params.token = authkey;
                        params.tripId = trip_id;
                        params.jobId = job_id;
                        params.lat = (pos == null ? 0 : pos.latitude);
                        params.lon = (pos == null ? 0 : pos.longitude);
                        params.name = options.fileName;
                        app.utils.debug.log(app.utils.debug.debugPages.SYNCHRONISATOR, "Bild uploaden: "+ params.name, "INFO", trip_id);
                        options.params = params;
                        console.log("PARAMS\n");
                        console.log(params);
                        var ft = new FileTransfer();
                        ft.upload(fileURL, encodeURI(baseUrl + ADD_IMAGE), function (r) {
                            console.log("Code = " + r.responseCode);
                            //console.log("Response = " + r.response);
                            console.log("Sent = " + r.bytesSent);
                            var result = JSON.parse(r.response);
                            console.log(result);
                            console.log(r.response.success);
                            console.log(r.response);
                            if (result.success) {
                                var query = "UPDATE image SET synced = 1 WHERE id=" + item.id;
                                console.log(result.data);
                                app.db.executeSql(query, [], function (r) {
                                    IN_IMAGE_SYNC = false;
                                });
                                console.log("Bildübertragung hat " + (new Date() - start) + " ms gedauert.");
                                app.utils.debug.log(app.utils.debug.debugPages.SYNCHRONISATOR, 
                                	"Bildübertragung hat " + (new Date() - start) + " ms gedauert.", "INFO", trip_id);
                            }
                            else {
                            	app.utils.debug.log(app.utils.debug.debugPages.SYNCHRONISATOR, "Bild nicht übertragen", "ERROR", trip_id);
                            	app.utils.debug.log(app.utils.debug.debugPages.SYNCHRONISATOR, result, "ERROR", trip_id);
                            }
                            IN_IMAGE_SYNC = false;
                        }, function (error) {
                            console.log(error);
                            console.log("upload error source " + error.source);
                            console.log("upload error target " + error.target);
                            console.log("IMAGE SYNC ERROR");
                            alert("Bild nicht übertragen!");
                            app.utils.debug.log(app.utils.debug.debugPages.SYNCHRONISATOR, "Bild nicht übertragen", "ERROR", trip_id);
                            app.utils.debug.log(app.utils.debug.debugPages.SYNCHRONISATOR, error, "ERROR", trip_id);     
                        }, options);
                    });
                });
            });
        }

        function syncNote(id) {
            app.db.executeSql("SELECT * FROM note WHERE synced = 0 AND trip_id = ? LIMIT 1", [id], function (result) {
                if (result.rows.length == 0) {
                    console.log("no notes for " + id);
                    return;
                }
                app.db.getLoginData(function (username, authkey) {
                    var item = result.rows.item(0),
                        note = item.note,
                        pos = item.posJsonObj;
                    console.log("TRYING TO SYNC NOTE: " + note);
                    console.log("ITEM: " + item);
                    IN_NOTE_SYNC = true;
                    app.db.getExternalJobIdAndExternalTripIdByInternalTripId(id, function (job_id, trip_id) {
                        console.log(job_id + " => " + trip_id);
                        $.ajax({
                            method: "POST",
                            url: baseUrl + ADD_NOTE,
                            data: {
                                user: username,
                                token: authkey,
                                tripId: trip_id,
                                jobId: job_id,
                                lat: 0,
                                lon: 0,
                                note: note,
                                creationDate: false
                            },
                            dataType: "json",
                            success: function (result) {
                                console.log(result);
                                app.db.executeSql("UPDATE note SET synced = 1 WHERE id = ?", [item.id], function (result) {
                                    IN_NOTE_SYNC = false;
                                });
                            },
                            error: function (e) {
                                console.log(e);
                                alert("Notiz nicht übertragen!");
                                app.utils.debug.log(app.utils.debug.debugPages.SYNCHRONISATOR, "Notiz nicht übertragen", "ERROR", trip_id);
                                app.utils.debug.log(app.utils.debug.debugPages.SYNCHRONISATOR, e, "ERROR", trip_id);
                                IN_NOTE_SYNC = false;
                            }
                        });
                    });
                });
            });
        }

        function syncWaypoint(id) {
            app.db.executeSql("SELECT * FROM position WHERE synced = 0 AND trip_id = ?", [id], function (result) {
                if (result.rows.length == 0) {
                    console.log("no result");
                    return;
                }
                app.db.getExternalJobIdAndExternalTripIdByInternalTripId(id, function (job_id, trip_id) {
                    IN_WAYPOINT_SYNC = true;
                    var data = [];
                    for (var i = 0; i < result.rows.length; i++) {
                        var line = result.rows.item(i);
                        var pos = $.parseJSON(line.posJsonObj);
                        var lineItem = {};
                        lineItem["jobId"] = job_id;
                        lineItem["tripId"] = trip_id;
                        lineItem["lon"] = pos.longitude;
                        lineItem["lat"] = pos.latitude;
                        lineItem["localId"] = line.id;
                        lineItem["timestamp"] = pos.timestamp;
                        data.push(lineItem);
                    }
                    app.db.getLoginData(function (username, authkey) {
                        var jData = JSON.stringify(data);
                        console.log(JSON.stringify(data), app.utils.debug.debugPages.SYNCHRONISATOR);
                        console.log("baseUrl: "+baseUrl);
                        $.ajax({
                            method: "POST",
                            url: baseUrl + ADD_WAY_POINTS,
                            data: {
                                user: username,
                                token: authkey,
                                data: jData
                            },
                            dataType: "json",
                            success: function (result) {
                                if (result.data.result) {
                                    var query = "UPDATE position SET synced = 1 WHERE id IN(";
                                    var and = "";
                                    $.each(result.data.result, function (key, value) {
                                        query += and;
                                        query += value;
                                        and = ", ";
                                    });
                                    query += ")";
                                    app.db.executeSql(query, [], function (result) {
                                        IN_WAYPOINT_SYNC = false;
                                    });
                                }
                            },
                            error: function (e) {
                                IN_WAYPOINT_SYNC = false;
                                alert("Wegpunkt nicht übertragen!");
                                app.utils.debug.log(app.utils.debug.debugPages.SYNCHRONISATOR, "Wegpunkt nicht übertragen", "ERROR", trip_id);
	                            app.utils.debug.log(app.utils.debug.debugPages.SYNCHRONISATOR, e, "ERROR", trip_id);
                            }
                        });
                    });
                });
            });
        }

        function getFileContentAsBase64(path, callback) {
            window.resolveLocalFileSystemURL(path, gotFile, fail);

            function fail(e) {
                alert('Cannot found requested file');
            }

            function gotFile(fileEntry) {
                fileEntry.file(function (file) {
                    var reader = new FileReader();
                    reader.onloadend = function (e) {
                        var content = this.result;
                        callback(content);
                    };
                    // The most important point, use the readAsDatURL Method from the file plugin
                    reader.readAsDataURL(file);
                });
            }
        }

        function stopSyncing() {
            window.clearInterval(timer);
        }

        function syncTripStart() {

        }

        function syncTripEnd() {

        }

        function synchronizer(url, data) {

        }

        //public
        return {
            startSynchronisation: function () {
                startSyncing();
            },
            stopSynchronisation: function () {
                stopSyncing();
            },
            synchronizeTripStart: function () {
                syncTripStart();
            },
            synchronizeTripEnd: function () {
                syncTripEnd();
            }
        }
    }

    return {
        getInstance: function () {
            if (!instance) {
                instance = init();
            }
            return instance;
        }
    }
})();
