/**
 * Created by christiankalig on 05.07.16.
 */

var app = app || {};

var Utils = (function(){
    var instance;

    function init(){
        console.log("Utils initialized");
        // private
        var logFiles = [];
        
        //public
        return {
        	startLog: function(tripId, cb) {
        		window.resolveLocalFileSystemURL(cordova.file.dataDirectory, function(dir) {
			        console.log("got main dir",dir);
			        var filename = 'log_'+tripId+'.txt';
			        dir.getFile("log_"+tripId+".txt", {create:true}, function(file) {
			            console.log("got the file", file);
			            if (tripId <= 0 && logFiles[tripId] === undefined) {
			            	//in case of trip 0, we're logging system information. Therefore, start a new file if not loaded yet
				            file.createWriter(function(fileWriter) {
				            	fileWriter.onwriteend= function(evt) {
							        fileWriter.seek(0);
							        fileWriter.write("File created");
							    }
								fileWriter.truncate(0);
								console.log("got a cleared file");
							}, function(err) {
								console.error(err);
							});
			            }
			            logFiles[tripId] = file;
			            cb(file);
			        });
		        });
        	},
            /**
             * returns a distance between 2 points defined by longitude and latitude
             * @param lat1
             * @param lon1
             * @param lat2
             * @param lon2
             * @returns {number}
             */
            measure: function(lat1, lon1, lat2, lon2){  // generally used geo measurement function
                var R = 6378.137; // Radius of earth in KM
                var dLat = (lat2 - lat1) * Math.PI / 180;
                var dLon = (lon2 - lon1) * Math.PI / 180;
                var a = Math.sin(dLat/2) * Math.sin(dLat/2) +
                    Math.cos(lat1 * Math.PI / 180) * Math.cos(lat2 * Math.PI / 180) *
                    Math.sin(dLon/2) * Math.sin(dLon/2);
                var c = 2 * Math.atan2(Math.sqrt(a), Math.sqrt(1-a));
                var d = R * c;
                console.log("Measure: "+(d * 1000)); // meters (i believe)
                return d * 1000;
            },
            /**
             * rounds a number
             * @param value
             * @param decimal
             * @returns {number}
             */
            round: function(value, decimal){
                return Math.round(value * Math.pow(10, decimal)) / Math.pow(10, decimal);
            },
            /**
             * formats the interval between two date objects to HH:MM:SS
             * @param begin
             * @param end
             * @returns {string}
             */
            timeIntervalToString: function(begin, end){
                if(end === false) end = new Date();
                var msec = end-begin;
                var hh = Math.floor(msec / 1000 / 60 / 60);
                msec -= hh * 1000 * 60 * 60;
                var mm = Math.floor(msec / 1000 / 60);
                msec -= mm * 1000 * 60;
                var ss = Math.floor(msec / 1000);
                msec -= ss * 1000;
                if(hh < 10) hh = "0"+hh;
                if(mm < 10) mm = "0"+mm;
                if(ss < 10) ss = "0"+ss;
                return hh+":"+mm+":"+ss;
            },
            debug: {
                debugger: true,
                debugPages: {
                    DASHBOARD: 1,
                    JOB_OVERVIEW: 2,
                    JOB_REVIEW: 3,
                    JOBS: 4,
                    JOB_TRACKER: 5,
                    LOGIN: 6,
                    PANEL: 7,
                    DB: 8,
                    EVENT_HANDLER: 9,
                    SYNCHRONISATOR: 10,
                    UTILS: 11,
                    TRIP_OVERVIEW: 12
                },
                debuggablePages: [10],
                writeLog: function(log, logOb) {
                	try {
						logOb.createWriter(function(fileWriter) {
							fileWriter.seek(fileWriter.length);					        
							var blob = new Blob([log], {type:'text/plain'});
							fileWriter.write(blob);
							console.log("logged");
						}, function(err) {
							console.error(err);
						});
					}
					catch (err) {
					   	console.error(err);
					}
                },
                log: function(page, msg, type, tripId){
                    if(this.debugger){
                    	if (type == null) {
                    		type = "INFO";
                    	}
                    	var log = " [" + (new Date()) + "] " + type + "-P" + (page > 0 ? page : 0) +": " + JSON.stringify(msg) +"\n";
					    console.log(log);
					    if (tripId == null) {
					    	//set current trip if available
					    	console.log("trip id not set, set to current trip");
					    	tripId = app.data.currentTripExtId;
					    }
					    if (tripId < 0) {
					    	tripId = 0;
					    }
					    //check whether file object already exists
					    if (logFiles[tripId] === undefined) {
					    	console.log("create log file first for trip " + tripId);
					    	//create file and log afterwards
					    	app.utils.startLog(tripId, function(logOb) {
					    		console.log("now write log");
					    		app.utils.debug.writeLog(log, logOb);
					    	});
					    }
					    else {
					    	console.log("write log directly");
					    	//log directly
					    	var logOb = logFiles[tripId];
					    	app.utils.debug.writeLog(log, logOb);
					    }
					    
                    }
                },
                getLog: function(tripId) {
                	if (tripId == null) {
                		//set current trip if available
                		console.log("trip id not set, set to current trip");
                		tripId = app.data.currentTripExtId;
                	}
                	if (logFiles[tripId] === undefined) {
                		console.error("no log for trip " + tripId);
                		return;
                	}
                	try {
                	var logOb = logFiles[tripId];
	                	logOb.file(function(file) {
					        var reader = new FileReader();
					        reader.onloadend = function(e) {
					            console.log(this.result);
					        };
					        reader.readAsText(file);
					    }, function(err) {
						   	console.error(err);
						});
                	}
                	catch (err) {
                		console.error(err);
                	}
                }
            }
        }
    }

    return {
        getInstance: function(){
            if(!instance){
                instance = init();
            }
            return instance;
        }
    }
})();